//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * プリミティブ描画リクエストビルダヘッダ
 * @author Junpee
 */

#ifndef PRIMITIVE_DRAW_REQUEST_BUILDER_H_
#define PRIMITIVE_DRAW_REQUEST_BUILDER_H_

namespace Lamp{

class PrimitiveDrawRequest;
class CharacterModel;

//------------------------------------------------------------------------------
/**
 * プリミティブ描画リクエストビルダ
 */
class PrimitiveDrawRequestBuilder{
public:
	/**
	 * ポイントの構築
	 * @param request 描画リクエスト
	 */
	static void buildPoint(PrimitiveDrawRequest* request);

	/**
	 * 軸ポイントの構築
	 * @param request 描画リクエスト
	 */
	static void buildAxisPoint(PrimitiveDrawRequest* request);

	/**
	 * 軸の構築
	 * @param request 描画リクエスト
	 */
	static void buildAxis(PrimitiveDrawRequest* request);

	/**
	 * 矢印の構築
	 * @param request 描画リクエスト
	 */
	static void buildArrow(PrimitiveDrawRequest* request);

	/**
	 * グリッドの構築
	 * @param request 描画リクエスト
	 * @param division 分割数
	 * @param subDivision サブ分割数
	 */
	static void buildGrid(PrimitiveDrawRequest* request,
		int division, int subDivision);

	/**
	 * 平面の構築
	 * @param request 描画リクエスト
	 */
	static void buildPlane(PrimitiveDrawRequest* request);

	/**
	 * 球の構築
	 * @param request 描画リクエスト
	 * @param radius 半径
	 * @param horizontalDivision 水平分割数
	 * @param verticalDivision 垂直分割数
	 */
	static void buildSphere(PrimitiveDrawRequest* request,
		float radius, int horizontalDivision, int verticalDivision);

	/**
	 * 箱の構築
	 * @param request 描画リクエスト
	 * @param width 幅
	 * @param height 高さ
	 * @param depth 奥行き
	 */
	static void buildBox(PrimitiveDrawRequest* request,
		float width, float height, float depth);

	/**
	 * 円柱の構築
	 * @param request 描画リクエスト
	 * @param radius 半径
	 * @param height 高さ
	 * @param division 分割数
	 */
	static void buildCylinder(PrimitiveDrawRequest* request,
		float radius, float height, int division);

	/**
	 * コーンの構築
	 * @param request 描画リクエスト
	 * @param radius 半径
	 * @param height 高さ
	 * @param division 分割数
	 */
	static void buildCone(PrimitiveDrawRequest* request,
		float radius, float height, int division);

	/**
	 * ボーンの構築
	 * @param request 描画リクエスト
	 * @param model キャラクタモデル
	 */
	static void buildBone(PrimitiveDrawRequest* request, CharacterModel* model);

private:
	//--------------------------------------------------------------------------
	// コンストラクタの隠蔽
	PrimitiveDrawRequestBuilder();

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of PRIMITIVE_DRAW_REQUEST_BUILDER_H_
//------------------------------------------------------------------------------
